@@ -70,11 +70,9 @@ pub(crate) enum FendError {
 	ParseDateError(String),
 	ParseError(crate::parser::ParseError),
 	ExpectedAString,
-	ExpComplex,
 	ExpectedARealNumber,
 	ConversionRhsNumerical,
 	FactorialUnitless,
-	RootsComplex,
 	ModuloForPositiveInts,
 	ExpUnitless,
 	IncompatibleConversion {
@@ -109,8 +107,6 @@ impl fmt::Display for FendError {
 			Self::DeserializationError => write!(f, "failed to deserialize object"),
 			Self::ModuloUnitless => write!(f, "modulo is only supported for unitless numbers"),
 			Self::FactorialComplex => write!(f, "factorial is not supported for complex numbers"),
-			Self::RootsComplex => write!(f, "roots are currently unsupported for complex numbers"),
-			Self::ExpComplex => write!(f, "exponentiation is not supported for complex numbers"),
 			Self::ExpUnitless => write!(f, "exponentiation is only supported for unitless numbers"),
 			Self::IoError(_) => write!(f, "I/O error"),
 			Self::InvalidBasePrefix => write!(
@@ -125,9 +121,9 @@ impl fmt::Display for FendError {
 				to_base,
 			} => {
 				write!(
-                    f,
-                    "cannot convert from {from} to {to}: units '{from_base}' and '{to_base}' are incompatible"
-                )
+					f,
+					"cannot convert from {from} to {to}: units '{from_base}' and '{to_base}' are incompatible"
+				)
 			}
 			Self::NonIntegerNegRoots => write!(f, "cannot compute non-integer or negative roots"),
 			Self::RootsOfNegativeNumbers => {
@@ -244,9 +240,9 @@ impl fmt::Display for FendError {
 				after,
 			} => {
 				write!(
-                    f,
-                    "{month} {expected_day}, {year} does not exist, did you mean {before} or {after}?",
-                )
+					f,
+					"{month} {expected_day}, {year} does not exist, did you mean {before} or {after}?",
+				)
 			}
 		}
 	}
