@@ -204,6 +204,7 @@ pub struct Context {
 	random_u32: Option<fn() -> u32>,
 	output_mode: OutputMode,
 	get_exchange_rate: Option<Arc<dyn ExchangeRateFn + Send + Sync>>,
+	custom_units: Vec<(String, String, String)>,
 }
 
 impl fmt::Debug for Context {
@@ -235,6 +236,7 @@ impl Context {
 			random_u32: None,
 			output_mode: OutputMode::SimpleText,
 			get_exchange_rate: None,
+			custom_units: vec![],
 		}
 	}
 
@@ -335,6 +337,42 @@ impl Context {
 	) {
 		self.get_exchange_rate = Some(Arc::new(get_exchange_rate));
 	}
+
+	pub fn define_custom_unit_v1(
+		&mut self,
+		singular: &str,
+		plural: &str,
+		definition: &str,
+		attribute: &CustomUnitAttribute,
+	) {
+		let definition_prefix = match attribute {
+			CustomUnitAttribute::None => "",
+			CustomUnitAttribute::AllowLongPrefix => "l@",
+			CustomUnitAttribute::AllowShortPrefix => "s@",
+			CustomUnitAttribute::IsLongPrefix => "lp@",
+			CustomUnitAttribute::Alias => "=",
+		};
+		self.custom_units.push((
+			singular.to_string(),
+			plural.to_string(),
+			format!("{definition_prefix}{definition}"),
+		));
+	}
+}
+
+/// These attributes make is possible to change the behaviour of custom units
+#[non_exhaustive]
+pub enum CustomUnitAttribute {
+	/// Don't allow using prefixes with this custom unit
+	None,
+	/// Support long prefixes (e.g. `milli-`, `giga-`) with this unit
+	AllowLongPrefix,
+	/// Support short prefixes (e.g. `k` for `kilo`) with this unit
+	AllowShortPrefix,
+	/// Allow using this unit as a long prefix with another unit
+	IsLongPrefix,
+	/// This unit definition is an alias and will always be replaced with its definition.
+	Alias,
 }
 
 /// This function evaluates a string using the given context. Any evaluation using this
@@ -444,14 +482,89 @@ impl Completion {
 	}
 }
 
+static GREEK_LOWERCASE_LETTERS: [(&str, &str); 24] = [
+	("alpha", "α"),
+	("beta", "β"),
+	("gamma", "γ"),
+	("delta", "δ"),
+	("epsilon", "ε"),
+	("zeta", "ζ"),
+	("eta", "η"),
+	("theta", "θ"),
+	("iota", "ι"),
+	("kappa", "κ"),
+	("lambda", "λ"),
+	("mu", "μ"),
+	("nu", "ν"),
+	("xi", "ξ"),
+	("omicron", "ο"),
+	("pi", "π"),
+	("rho", "ρ"),
+	("sigma", "σ"),
+	("tau", "τ"),
+	("upsilon", "υ"),
+	("phi", "φ"),
+	("chi", "χ"),
+	("psi", "ψ"),
+	("omega", "ω"),
+];
+static GREEK_UPPERCASE_LETTERS: [(&str, &str); 24] = [
+	("Alpha", "Α"),
+	("Beta", "Β"),
+	("Gamma", "Γ"),
+	("Delta", "Δ"),
+	("Epsilon", "Ε"),
+	("Zeta", "Ζ"),
+	("Eta", "Η"),
+	("Theta", "Θ"),
+	("Iota", "Ι"),
+	("Kappa", "Κ"),
+	("Lambda", "Λ"),
+	("Mu", "Μ"),
+	("Nu", "Ν"),
+	("Xi", "Ξ"),
+	("Omicron", "Ο"),
+	("Pi", "Π"),
+	("Rho", "Ρ"),
+	("Sigma", "Σ"),
+	("Tau", "Τ"),
+	("Upsilon", "Υ"),
+	("Phi", "Φ"),
+	("Chi", "Χ"),
+	("Psi", "Ψ"),
+	("Omega", "Ω"),
+];
+
 #[must_use]
 pub fn get_completions_for_prefix(mut prefix: &str) -> (usize, Vec<Completion>) {
+	if let Some((prefix, letter)) = prefix.rsplit_once('\\') {
+		if letter.starts_with(|c: char| c.is_ascii_alphabetic()) && letter.len() <= 7 {
+			return if letter.starts_with(|c: char| c.is_ascii_uppercase()) {
+				GREEK_UPPERCASE_LETTERS
+			} else {
+				GREEK_LOWERCASE_LETTERS
+			}
+			.iter()
+			.find(|l| l.0 == letter)
+			.map_or((0, vec![]), |l| {
+				(
+					prefix.len(),
+					vec![Completion {
+						display: prefix.to_string(),
+						insert: l.1.to_string(),
+					}],
+				)
+			});
+		}
+	}
+
 	let mut prepend = "";
 	let position = prefix.len();
 	if let Some((a, b)) = prefix.rsplit_once(' ') {
 		prepend = a;
 		prefix = b;
 	}
+
 	if prefix.is_empty() {
 		return (0, vec![]);
 	}
