@@ -9,6 +9,7 @@ use std::fmt::Write;
 use std::ops::Neg;
 use std::{fmt, io};
 
+use super::real::Real;
 use super::{Base, Exact, FormattingStyle};
 
 #[derive(Clone)]
@@ -269,6 +270,12 @@ impl From<Complex> for Dist {
 	}
 }
 
+impl From<Real> for Dist {
+	fn from(v: Real) -> Self {
+		Self::from(Complex::from(v))
+	}
+}
+
 impl From<u64> for Dist {
 	fn from(i: u64) -> Self {
 		Self::from(Complex::from(i))
