@@ -1,3 +1,5 @@
+use std::borrow::Cow;
+
 #[derive(Eq, PartialEq, PartialOrd, Ord, Clone, Copy)]
 struct UnitDef {
 	singular: &'static str,
@@ -504,6 +506,35 @@ const COMMON_PHYSICAL_UNITS: &[UnitTuple] = &[
 	("light", "", "c", ""),
 ];
 
+const CGS_UNITS: &[UnitTuple] = &[
+	("gal", "gals", "cm/s^2", "acceleration"),
+	("dyne", "dynes", "g*gal", "force"),
+	("erg", "ergs", "g*cm^2/s^2", "work, energy"),
+	("barye", "baryes", "g/(cm*s^2)", "pressure"),
+	("poise", "poises", "g/(cm*s)", ""),
+	("stokes", "", "cm^2/s", ""),
+	("kayser", "kaysers", "cm^-1", ""),
+	("biot", "biots", "10 amperes", ""),
+	("emu", "emus", "0.001 A m^2", ""),
+	("franklin", "franklins", "dyn^1/2*cm", ""),
+	("gauss", "", "10^-4 tesla", ""),
+	("maxwell", "maxwells", "10^-8 weber", ""),
+	("phot", "phots", "10000 lux", ""),
+	("stilb", "stilbs", "10000 candela/m^2", ""),
+	// abbrevations
+	("gallileo", "gallileos", "gal", ""),
+	("dyn", "dyns", "dyne", ""),
+	("Ba", "", "barye", ""),
+	("P", "", "poise", ""),
+	("St", "", "stokes", ""),
+	("K", "", "kayser", ""),
+	("Bi", "", "biot", ""),
+	("Fr", "", "franklin", ""),
+	("G", "", "gauss", ""),
+	("Mx", "", "maxwell", ""),
+	("ph", "", "phot", ""),
+];
+
 const IMPERIAL_UNITS: &[UnitTuple] = &[
 	("inch", "inches", "2.54 cm", ""),
 	("mil", "mils", "1/1000 inch", ""),
@@ -516,12 +547,13 @@ const IMPERIAL_UNITS: &[UnitTuple] = &[
 	("yard", "yards", "l@3 ft", ""),
 	("mile", "miles", "l@5280 ft", ""),
 	("line", "lines", "1/12 inch", ""),
-	("rod", "", "5.5 yard", ""),
-	("pole", "", "rod", ""),
-	("perch", "", "rod", ""),
-	("furlong", "", "40 rod", ""),
+	("rod", "rods", "5.5 yard", ""),
+	("pole", "poles", "rod", ""),
+	("perch", "perches", "rod", ""),
+	("firkin", "firkins", "90 lb", ""),
+	("furlong", "furlongs", "40 rod", ""),
 	("statute_mile", "statute_miles", "mile", ""),
-	("league", "", "3 mile", ""),
+	("league", "leagues", "3 mile", ""),
 	("chain", "chains", "66 feet", ""),
 	("link", "links", "1/100 chain", ""),
 	("thou", "", "1/1000 inch", "thousandth of an inch"),
@@ -532,7 +564,7 @@ const IMPERIAL_UNITS: &[UnitTuple] = &[
 	("point", "points", "l@1/72 inch", ""),
 	("twip", "twips", "l@1/20 point", ""),
 	("poppyseed", "poppyseeds", "l@line", ""),
-	("pica", "", "l@12 points", ""),
+	("pica", "picas", "l@12 points", ""),
 	("barleycorn", "barleycorns", "l@4 poppyseed", ""),
 	("finger", "fingers", "l@63 points", ""),
 	("stick", "sticks", "l@2 inches", ""),
@@ -543,16 +575,16 @@ const IMPERIAL_UNITS: &[UnitTuple] = &[
 	("hand", "hands", "l@2 sticks", ""),
 	("shaftment", "shaftments", "l@2 palm", ""),
 	("cubit", "cubits", "l@2 span", ""),
-	("ell", "", "l@5 span", ""),
-	("skein", "", "l@96 ell", ""),
-	("spindle", "", "l@120 skein", ""),
+	("ell", "ells", "l@5 span", ""),
+	("skein", "skeins", "l@96 ell", ""),
+	("spindle", "spindles", "l@120 skein", ""),
 	("link", "links", "l@1/25 rod", ""),
 	("fathom", "fathoms", "l@2 yard", ""),
 	("shackle", "shackles", "l@15 yard", ""),
 	("pace", "paces", "l@5 shaftments", ""),
 	("step", "steps", "l@2 paces", ""),
-	("grade", "", "l@pace", ""),
-	("rope", "", "l@4 steps", ""),
+	("grade", "grades", "l@pace", ""),
+	("rope", "ropes", "l@4 steps", ""),
 	("ramsdens_chain", "", "l@5 rope", ""),
 	("roman_mile", "roman_miles", "l@50 ramsdens_chain", ""),
 	("gunters_chain", "gunters_chains", "l@4 rod", ""),
@@ -632,6 +664,9 @@ const IMPERIAL_ABBREVIATIONS: &[UnitTuple] = &[
 	("plf", "", "lb / foot", "pounds per linear foot"),
 	("lbf", "", "lb force", ""),
 	("psi", "", "pound force / inch^2", ""),
+	("fur", "furs", "furlong", ""),
+	("fir", "firs", "firkin", ""),
+	("ftn", "ftns", "fortnight", ""),
 ];
 
 const NAUTICAL_UNITS: &[UnitTuple] = &[
@@ -703,6 +738,7 @@ pub(crate) const ALL_UNIT_DEFS: &[&[UnitTuple]] = &[
 	IMPERIAL_ABBREVIATIONS,
 	NAUTICAL_UNITS,
 	CURRENCIES,
+	CGS_UNITS,
 ];
 
 const SHORT_PREFIXES: &[(&str, &str)] = &[
@@ -743,11 +779,11 @@ pub(crate) fn query_unit(
 	ident: &str,
 	short_prefixes: bool,
 	case_sensitive: bool,
-) -> Option<(&'static str, &'static str, &'static str)> {
+) -> Option<(Cow<'static, str>, Cow<'static, str>, Cow<'static, str>)> {
 	if short_prefixes {
 		for (name, def) in SHORT_PREFIXES {
 			if *name == ident || (!case_sensitive && name.eq_ignore_ascii_case(ident)) {
-				return Some((name, name, def));
+				return Some((Cow::Borrowed(name), Cow::Borrowed(name), Cow::Borrowed(def)));
 			}
 		}
 	}
@@ -760,7 +796,11 @@ pub(crate) fn query_unit(
 		.as_str(),
 	) {
 		let name = CURRENCY_IDENTIFIERS[idx];
-		return Some((name, name, "$CURRENCY"));
+		return Some((
+			Cow::Borrowed(name),
+			Cow::Borrowed(name),
+			Cow::Borrowed("$CURRENCY"),
+		));
 	}
 	let mut candidates = vec![];
 	for group in ALL_UNIT_DEFS {
@@ -771,18 +811,26 @@ pub(crate) fn query_unit(
 				definition: def.2,
 			};
 			if def.singular == ident || def.plural == ident {
-				return Some((def.singular, def.plural, def.definition));
+				return Some((
+					Cow::Borrowed(def.singular),
+					Cow::Borrowed(def.plural),
+					Cow::Borrowed(def.definition),
+				));
 			}
 			if !case_sensitive
 				&& (def.singular.eq_ignore_ascii_case(ident)
 					|| def.plural.eq_ignore_ascii_case(ident))
 			{
-				candidates.push(Some((def.singular, def.plural, def.definition)));
+				candidates.push(Some((
+					Cow::Borrowed(def.singular),
+					Cow::Borrowed(def.plural),
+					Cow::Borrowed(def.definition),
+				)));
 			}
 		}
 	}
 	if candidates.len() == 1 {
-		return candidates[0];
+		return candidates.into_iter().next().unwrap();
 	}
 	None
 }
